@@ -30,9 +30,6 @@ typedef struct
     PyObject_HEAD
 } pysqlite_PrepareProtocol;
 
-extern PyTypeObject *pysqlite_PrepareProtocolType;
-
 int pysqlite_prepare_protocol_setup_types(PyObject *module);
 
-#define UNKNOWN (-1)
 #endif
